@@ -13,6 +13,8 @@ from requests.auth import HTTPBasicAuth
 from crosspm.adapters.common import BaseAdapter
 from crosspm.helpers.exceptions import *  # noqa
 from crosspm.helpers.package import Package
+from crosspm.helpers.config import CROSSPM_DEPENDENCY_LOCK_FILENAME
+
 
 CHUNK_SIZE = 1024
 
@@ -271,6 +273,8 @@ class Adapter(BaseAdapter):
                 if downloader.do_load:
                     _package.download()
                     _deps_file = _package.get_file(self._config.deps_lock_file_name)
+                    if not _deps_file:
+                        _deps_file = _package.get_file(CROSSPM_DEPENDENCY_LOCK_FILENAME)
                     if downloader.recursive:
                         if _deps_file:
                             _package.find_dependencies(_deps_file, property_validate=False)
