@@ -1,16 +1,12 @@
 # -*- coding: utf-8 -*-
 import json
 import shutil
-from datetime import datetime
-import time
-
 import pathlib
-
 from crosspm.adapters.common import BaseAdapter
-from crosspm.helpers.exceptions import *
+from crosspm.helpers.exceptions import *  # noqa
 from crosspm.helpers.package import Package
+from crosspm.helpers.config import CROSSPM_DEPENDENCY_LOCK_FILENAME
 import os
-import requests
 
 CHUNK_SIZE = 1024
 
@@ -108,71 +104,82 @@ class Adapter(BaseAdapter):
         for _paths in parser.get_paths(list_or_file_path, source):
             _packages = []
             _params_found = {}
+            _params_found_raw = {}
+            last_error = ''
             _pkg_name = _paths['params'][_pkg_name_col]
             if _pkg_name != _pkg_name_old:
                 _pkg_name_old = _pkg_name
                 self._log.info(
-                    '{}: {}'.format(_pkg_name, {k: v for k, v in _paths['params'].items() if k != _pkg_name_col}))
-            last_error = ''
-            for _path in _paths['paths']:
-                _path_fixed, _path_pattern = parser.split_fixed_pattern(_path)
-                _repo_paths = FilesPath(_path_fixed)
-                try:
-                    for _repo_path in _repo_paths.glob(_path_pattern):
-                        _mark = 'found'
-                        _matched, _params = parser.validate_path(str(_repo_path), _paths['params'])
-                        if _matched:
-                            _params_found[_repo_path] = {k: v for k, v in _params.items()}
-                            _mark = 'match'
-                            _valid, _params = parser.validate(_repo_path.properties, 'properties', _paths['params'],
-                                                              return_params=True)
-                            if _valid:
-                                _mark = 'valid'
-                                _packages += [_repo_path]
-                                _params_found[_repo_path].update({k: v for k, v in _params.items()})
-                                _params_found[_repo_path]['filename'] = str(_repo_path.name)
-                        self._log.debug('  {}: {}'.format(_mark, str(_repo_path)))
-                except RuntimeError as e:
+                    '{}: {}'.format(_pkg_name,
+                                    {k: v for k, v in _paths['params'].items() if
+                                     k not in (_pkg_name_col, 'repo')}))
+            for _sub_paths in _paths['paths']:
+                self._log.info('repo: {}'.format(_sub_paths['repo']))
+                for _path in _sub_paths['paths']:
+                    _tmp_params = dict(_paths['params'])
+                    _tmp_params['repo'] = _sub_paths['repo']
+
+                    _path_fixed, _path_pattern = parser.split_fixed_pattern(_path)
+                    _repo_paths = FilesPath(_path_fixed)
                     try:
-                        err = json.loads(e.args[0])
-                    except:
-                        err = {}
-                    if isinstance(err, dict):
-                        # TODO: Check errors
-                        # e.args[0] = '''{
-                        #                   "errors" : [ {
-                        #                     "status" : 404,
-                        #                     "message" : "Not Found"
-                        #                   } ]
-                        #                 }'''
-                        for error in err.get('errors', []):
-                            err_status = error.get('status', -1)
-                            err_msg = error.get('message', '')
-                            if err_status == 401:
-                                msg = 'Authentication error[{}]{}'.format(err_status,
-                                                                          (': {}'.format(err_msg)) if err_msg else '')
-                            elif err_status == 404:
-                                msg = last_error
-                            else:
-                                msg = 'Error[{}]{}'.format(err_status,
-                                                           (': {}'.format(err_msg)) if err_msg else '')
-                            if last_error != msg:
-                                self._log.error(msg)
-                            last_error = msg
+                        for _repo_path in _repo_paths.glob(_path_pattern):
+                            _mark = 'found'
+                            _matched, _params, _params_raw = parser.validate_path(str(_repo_path), _tmp_params)
+                            if _matched:
+                                _params_found[_repo_path] = {k: v for k, v in _params.items()}
+                                _params_found_raw[_repo_path] = {k: v for k, v in _params_raw.items()}
+                                _mark = 'match'
+                                _valid, _params = parser.validate(_repo_path.properties, 'properties', _tmp_params,
+                                                                  return_params=True)
+                                if _valid:
+                                    _mark = 'valid'
+                                    _packages += [_repo_path]
+                                    _params_found[_repo_path].update({k: v for k, v in _params.items()})
+                                    _params_found[_repo_path]['filename'] = str(_repo_path.name)
+                            self._log.debug('  {}: {}'.format(_mark, str(_repo_path)))
+                    except RuntimeError as e:
+                        try:
+                            err = json.loads(e.args[0])
+                        except Exception:
+                            err = {}
+                        if isinstance(err, dict):
+                            # TODO: Check errors
+                            # e.args[0] = '''{
+                            #                   "errors" : [ {
+                            #                     "status" : 404,
+                            #                     "message" : "Not Found"
+                            #                   } ]
+                            #                 }'''
+                            for error in err.get('errors', []):
+                                err_status = error.get('status', -1)
+                                err_msg = error.get('message', '')
+                                if err_status == 401:
+                                    msg = 'Authentication error[{}]{}'.format(err_status,
+                                                                              (': {}'.format(
+                                                                                  err_msg)) if err_msg else '')
+                                elif err_status == 404:
+                                    msg = last_error
+                                else:
+                                    msg = 'Error[{}]{}'.format(err_status,
+                                                               (': {}'.format(err_msg)) if err_msg else '')
+                                if last_error != msg:
+                                    self._log.error(msg)
+                                last_error = msg
 
             _package = None
             if _packages:
                 _packages = parser.filter_one(_packages, _paths['params'], _params_found)
-                if type(_packages) is dict:
+                if isinstance(_packages, dict):
                     _packages = [_packages]
 
                 if len(_packages) == 1:
                     _stat_pkg = self.pkg_stat(_packages[0]['path'])
 
+                    _params_raw = _params_found_raw.get(_packages[0]['path'], {})
                     _params_tmp = _params_found.get(_packages[0]['path'], {})
                     _params_tmp.update({k: v for k, v in _packages[0]['params'].items() if k not in _params_tmp})
                     _package = Package(_pkg_name, _packages[0]['path'], _paths['params'], downloader, self, parser,
-                                       _params_tmp, _stat_pkg)
+                                       _params_tmp, _params_raw, _stat_pkg)
                     _mark = 'chosen'
                     self._log.info('  {}: {}'.format(_mark, str(_packages[0]['path'])))
 
@@ -193,7 +200,7 @@ class Adapter(BaseAdapter):
             else:
                 _added = False
             if _package is not None:
-                _pkg_name = _package.get_name_and_path(True)
+                _pkg_name = _package.name
             if _added or (_package is not None):
                 if (_package is not None) or (not self._config.no_fails):
                     if (_package is not None) or (_packages_found.get(_pkg_name, None) is None):
@@ -202,18 +209,20 @@ class Adapter(BaseAdapter):
             if _added and (_package is not None):
                 if downloader.do_load:
                     _package.download()
-
-                    _deps_file = _package.get_file(self._config.deps_lock_file_name, downloader.cache.temp_path)
+                    _deps_file = _package.get_file(self._config.deps_lock_file_name)
+                    if not _deps_file:
+                        _deps_file = _package.get_file(CROSSPM_DEPENDENCY_LOCK_FILENAME)
                     if _deps_file:
                         _package.find_dependencies(_deps_file)
                     elif self._config.deps_file_name:
-                        _deps_file = _package.get_file(self._config.deps_file_name, downloader.cache.temp_path)
+                        _deps_file = _package.get_file(self._config.deps_file_name)
                         if _deps_file and os.path.isfile(_deps_file):
                             _package.find_dependencies(_deps_file)
 
         return _packages_found
 
-    def pkg_stat(self, package):
+    @staticmethod
+    def pkg_stat(package):
         _stat_attr = {'ctime': 'st_atime',
                       'mtime': 'st_mtime',
                       'size': 'st_size'}
@@ -244,12 +253,14 @@ class Adapter(BaseAdapter):
 
         return dest_path
 
-    def get_package_filename(self, package):
+    @staticmethod
+    def get_package_filename(package):
         if isinstance(package, FilesPath):
             return package.name
         return ''
 
-    def get_package_path(self, package):
+    @staticmethod
+    def get_package_path(package):
         if isinstance(package, FilesPath):
             return str(package)
         return ''
