@@ -1,15 +1,19 @@
 # -*- coding: utf-8 -*-
-import os
-import logging
 import json
+import logging
+import os
 import platform
+import sys
+
+import requests
 import yaml
+
+from crosspm.helpers.cache import Cache
 from crosspm.helpers.exceptions import *
 from crosspm.helpers.parser import Parser
 from crosspm.helpers.source import Source
-from crosspm.helpers.cache import Cache
 
-from requests.packages.urllib3 import disable_warnings
+requests.packages.urllib3.disable_warnings()  # noqa
 
 WINDOWS = (platform.system().lower() == 'windows') or (os.name == 'nt')
 DEFAULT_CONFIG_FILE = ('crosspm.yaml', 'crosspm.yml', 'crosspm.json',)
@@ -39,61 +43,69 @@ GLOBAL_CONFIG_PATH = [
 
 ENVIRONMENT_CONFIG_PATH = 'CROSSPM_CONFIG_PATH'
 CROSSPM_DEPENDENCY_FILENAME = 'dependencies.txt'  # maybe 'cpm.manifest'
-CROSSPM_DEPENDENCY_LOCK_FILENAME = CROSSPM_DEPENDENCY_FILENAME  # 'dependencies.txt.lock'
+CROSSPM_DEPENDENCY_LOCK_FILENAME = CROSSPM_DEPENDENCY_FILENAME + ".lock"  # 'dependencies.txt.lock'
 CROSSPM_ADAPTERS_NAME = 'adapters'
 CROSSPM_ADAPTERS_DIR = os.path.join(CROSSPM_ROOT_DIR, CROSSPM_ADAPTERS_NAME)
 
-disable_warnings()
 
-
-class Config(object):
-    _config_path_env = []
-    _sources = []
-    _adapters = {}
-    _parsers = {}
-    _defaults = {}
-    _columns = []
-    _not_columns = {}
-    _options = {}
-    _values = {}
-    _output = {}
-    _solid = {}
-    _fails = {}
-    cache = None
-    no_fails = False
-    name_column = ''
-    deps_file_name = ''
-    deps_lock_file_name = ''
+class Config:
     windows = WINDOWS
-    crosspm_cache_root = ''
-    depslock_path = ''
 
-    def __init__(self, config_file_name='', cmdline='', no_fails=False, depslock_path='', deps_path=''):
+    def __init__(self, config_file_name='', cmdline='', no_fails=False, deps_lock_file_path='', deps_file_path='',
+                 lock_on_success=False, prefer_local=False, deps_content=None, deps_lock_content=None, recursive=None):
         self._log = logging.getLogger('crosspm')
+        self._config_path_env = []
+        self._sources = []
+        self._adapters = {}
+        self._parsers = {}
+        self._defaults = {}
+        self._columns = []
+        self._not_columns = {}
+        self._options = {}
+        self._values = {}
+        self._output = {}
+        self._solid = {}
+        self._fails = {}
+        self.name_column = ''
+        self.deps_file_name = ''
+        self.deps_lock_file_name = ''
+        self.deps_file_path = ''
+        self.deps_lock_file_path = ''
+        self.lock_on_success = lock_on_success
+        self.prefer_local = prefer_local
+        self.recursive = recursive
+        self.crosspm_cache_root = ''
+        self.deps_content = deps_content
+        self.deps_lock_content = deps_lock_content
+
+        self.cache_config = {}
         self.init_env_config_path()
+        self.secret_variables = []
 
         cpm_conf_name = ''
-        if deps_path:
-            deps_path = deps_path.strip().strip('"').strip("'")
-            self.deps_path = os.path.realpath(os.path.expanduser(deps_path))
+
+        if deps_file_path:
+            self.deps_file_path = self._normalize_path(deps_file_path)
             if not cpm_conf_name:
-                cpm_conf_name = self.get_cpm_conf_name(deps_path)
-            if os.path.isfile(deps_path):
-                config_path_tmp = os.path.dirname(deps_path)
+                cpm_conf_name = self.get_cpm_conf_name(self.deps_file_path)
+            if not os.path.isdir(self.deps_file_path):
+                config_path_tmp = os.path.dirname(self.deps_file_path)
             else:
-                config_path_tmp = deps_path
+                config_path_tmp = self.deps_file_path
+                self.deps_file_path = os.path.join(self.deps_file_path, CROSSPM_DEPENDENCY_FILENAME)
             if config_path_tmp not in DEFAULT_CONFIG_PATH:
                 DEFAULT_CONFIG_PATH.append(config_path_tmp)
+            self.deps_lock_file_path = self.deps_file_path + '.lock'
 
-        if depslock_path:
-            depslock_path = depslock_path.strip().strip('"').strip("'")
-            self.depslock_path = os.path.realpath(os.path.expanduser(depslock_path))
+        if deps_lock_file_path:
+            self.deps_lock_file_path = self._normalize_path(deps_lock_file_path)
             if not cpm_conf_name:
-                cpm_conf_name = self.get_cpm_conf_name(depslock_path)
-            if os.path.isfile(depslock_path):
-                config_path_tmp = os.path.dirname(depslock_path)
+                cpm_conf_name = self.get_cpm_conf_name(self.deps_lock_file_path)
+            if not os.path.isdir(self.deps_lock_file_path):
+                config_path_tmp = os.path.dirname(self.deps_lock_file_path)
             else:
-                config_path_tmp = depslock_path
+                config_path_tmp = deps_lock_file_path
+                self.deps_lock_file_path = os.path.join(self.deps_lock_file_path, CROSSPM_DEPENDENCY_LOCK_FILENAME)
             if config_path_tmp not in DEFAULT_CONFIG_PATH:
                 DEFAULT_CONFIG_PATH.append(config_path_tmp)
 
@@ -107,7 +119,7 @@ class Config(object):
                     _override = False
             try:
                 _override = bool(_override)
-            except:
+            except Exception:
                 _override = True
         else:
             config_data = {}
@@ -118,18 +130,46 @@ class Config(object):
         else:
             config_data.update(self.read_config_file())
 
+        # Add secret variables to special list in config
+        if 'options' in config_data:
+            for variable, property in config_data['options'].items():
+                if property.get('secret', False):
+                    self.secret_variables.append(variable)
+
         self.no_fails = no_fails
         self.parse_config(config_data, cmdline)
-        self.cache = Cache(self, self.cache)
+        self.cache = Cache(self, self.cache_config)
         # self._fails = {}
 
+    @property
+    def deps_file_name(self):
+        return os.path.basename(self.deps_file_path)
+
+    @property
+    def deps_lock_file_name(self):
+        return os.path.basename(self.deps_lock_file_path)
+
+    @deps_file_name.setter
+    def deps_file_name(self, file_name):
+        self.deps_file_path = self._normalize_path(file_name)
+
+    @deps_lock_file_name.setter
+    def deps_lock_file_name(self, file_name):
+        self.deps_lock_file_path = self._normalize_path(file_name)
+
+    @staticmethod
+    def _normalize_path(raw_path):
+        _path = raw_path.strip('"').strip("'")
+        _path = os.path.realpath(os.path.expanduser(_path))
+        return _path
+
     def get_cpm_conf_name(self, deps_filename=''):
         if not deps_filename:
-            deps_filename = self.depslock_path
+            deps_filename = self.deps_file_path
         result = ''
         if os.path.isfile(deps_filename):
             try:
-                with open(deps_filename, 'r') as f:
+                with open(deps_filename, 'r', encoding="utf-8-sig") as f:
                     for line in f:
                         line = line.strip()
                         if line.startswith('#'):
@@ -138,7 +178,7 @@ class Config(object):
                                 if line[0].lower() == 'cpmconfig':
                                     result = line[1].split('#')[0].strip('"').strip("'")
                                     break
-            except:
+            except Exception:
                 pass
         return result
 
@@ -169,7 +209,7 @@ class Config(object):
         for config_path in GLOBAL_CONFIG_PATH:
             try:
                 _path = config_path.format(**args).strip().strip("'").strip('"')
-            except:
+            except Exception:
                 _path = ''
             if _path:
                 for config_name in GLOBAL_CONFIG_FILE:
@@ -178,7 +218,8 @@ class Config(object):
                         return _file
         return ''
 
-    def find_cpmconfig(self, conf_name=''):
+    @staticmethod
+    def find_cpmconfig(conf_name=''):
         conf_path_add = ''
         try:
             _temp = __import__('cpmconfig.cpmconfig', globals(), locals(), ['App'], 0)
@@ -196,7 +237,7 @@ class Config(object):
             if conf_name:
                 conf_path_add = cpm_find(None, conf_name)[2]
 
-        except:
+        except Exception:
             conf_path = ''
         if conf_path and not os.path.isfile(conf_path):
             conf_path = ''
@@ -218,7 +259,7 @@ class Config(object):
             if cpm_file_deps:
                 DEFAULT_CONFIG_PATH.insert(ind, cpm_file_deps)
 
-            _def_conf_file = [DEFAULT_CONFIG_FILE] if type(DEFAULT_CONFIG_FILE) is str else DEFAULT_CONFIG_FILE
+            _def_conf_file = [DEFAULT_CONFIG_FILE] if isinstance(DEFAULT_CONFIG_FILE, str) else DEFAULT_CONFIG_FILE
             for config_path in DEFAULT_CONFIG_PATH:
                 config_path = config_path.strip().strip("'").strip('"')
                 config_path = os.path.realpath(os.path.expanduser(config_path))
@@ -269,8 +310,8 @@ class Config(object):
         elif _ext == '.json':
             _is_yaml = False
         else:
-            with open(_config_file_name) as f:
-                for i, line in enumerate(f):
+            with open(_config_file_name, encoding="utf-8-sig") as f:
+                for line in f:
                     line = line.strip()
                     if not line:
                         continue
@@ -282,7 +323,7 @@ class Config(object):
             if _is_yaml:
                 result = self.load_yaml(_config_file_name)
             else:
-                with open(_config_file_name) as f:
+                with open(_config_file_name, encoding="utf-8-sig") as f:
                     result = json.loads(f.read())
 
         except Exception as e:
@@ -293,7 +334,8 @@ class Config(object):
 
         return result
 
-    def find_import_file(self, import_file_name=''):
+    @staticmethod
+    def find_import_file(import_file_name=''):
         res_file_name = ''
         if import_file_name:
             for config_path in [''] + DEFAULT_CONFIG_PATH:
@@ -319,12 +361,15 @@ class Config(object):
 
         return res_file_name
 
-    def load_yaml(self, _config_file_name):
+    @staticmethod
+    def load_yaml(_config_file_name):
         result = {}
         yaml_imports = ''
         yaml_content = ''
-        with open(_config_file_name) as f:
+        with open(_config_file_name, encoding="utf-8-sig") as f:
             for line in f:
+                if line.strip().startswith('#'):
+                    continue
                 if (not yaml_imports) and (not yaml_content):
                     line_one = line.strip().replace(' ', '').lower()
                     if line_one.startswith('import:'):
@@ -342,13 +387,13 @@ class Config(object):
             data_imports = yaml.safe_load(yaml_imports)
             yaml_content_pre = ''
             if 'import' in data_imports:
-                if type(data_imports['import']) is str:
+                if isinstance(data_imports['import'], str):
                     data_imports['import'] = [data_imports['import']]
-                if type(data_imports['import']) in (list, tuple):
+                if isinstance(data_imports['import'], (list, tuple)):
                     for _import_file_name in data_imports['import']:
-                        _import_file_name = self.find_import_file(_import_file_name)
+                        _import_file_name = Config.find_import_file(_import_file_name)
                         if _import_file_name:
-                            with open(_import_file_name) as f:
+                            with open(_import_file_name, encoding="utf-8-sig") as f:
                                 for line in f:
                                     yaml_content_pre += line
                             yaml_content_pre += '\n'
@@ -384,7 +429,8 @@ class Config(object):
         # init default values for columns
         if 'defaults' in config_data:
             self._defaults = config_data['defaults']
-        self._defaults.update({k: v['default'] for k, v in self._options.items() if k not in self._defaults})
+        self._defaults.update(
+            {k: v['default'] for k, v in self._options.items() if k not in self._defaults and 'default' in v})
 
         # init solid
         if 'solid' in config_data:
@@ -456,7 +502,7 @@ class Config(object):
 
         try:
             _cmdline = {x[0]: x[1] for x in [x.strip().split('=') for x in cmdline.split(',')] if len(x) > 1}
-        except:
+        except Exception:
             _cmdline = {}
 
         # init cpm parameters
@@ -476,6 +522,18 @@ class Config(object):
     def init_cpm_and_cache(self, crosspm, cmdline, cache_config):
         if 'cache' not in crosspm:
             crosspm['cache'] = {}
+
+        # have to merge dicts:
+        parts = ('clear', 'storage')
+        cache_config.update(
+            {k: v for k, v in crosspm['cache'].items() if (k not in cache_config) and (k not in parts)})
+        for part in parts:
+            if part in crosspm['cache']:
+                if part not in cache_config:
+                    cache_config[part] = {}
+                cache_config[part].update(
+                    {k: v for k, v in crosspm['cache'][part].items() if k not in cache_config[part]})
+
         if 'path' in cache_config:
             crosspm['cache'] = cache_config.pop('path')
         else:
@@ -485,11 +543,29 @@ class Config(object):
 
         crosspm = self.parse_options(crosspm, cmdline)
 
-        self.deps_lock_file_name = crosspm.get('dependencies-lock', CROSSPM_DEPENDENCY_LOCK_FILENAME)
-        if 'dependencies' in crosspm:
+        if not self.deps_file_name:
             self.deps_file_name = crosspm.get('dependencies', CROSSPM_DEPENDENCY_FILENAME)
-            if 'dependencies-lock' not in crosspm:
+        if not self.deps_lock_file_name:
+            self.deps_lock_file_name = crosspm.get('dependencies-lock', CROSSPM_DEPENDENCY_LOCK_FILENAME)
+            if 'dependencies-lock' not in crosspm and self.deps_file_name.endswith(".lock"):
                 self.deps_lock_file_name = self.deps_file_name
+            elif 'dependencies-lock' not in crosspm:
+                self.deps_lock_file_name = self.deps_file_name + ".lock"
+
+        if self.recursive is None:
+            self.recursive = crosspm.get('recursive', False)
+
+        if not self.prefer_local:
+            self.prefer_local = crosspm.get('prefer-local', False)
+        if not self.lock_on_success:
+            lock_on_success = crosspm.get('lock-on-success', False)
+            if isinstance(lock_on_success, str):
+                if lock_on_success.lower() in ['1', 'yes', '+', 'true']:
+                    self.lock_on_success = True
+            try:
+                self.lock_on_success = bool(lock_on_success)
+            except Exception:
+                self.lock_on_success = False
 
         # Cache init
         self.crosspm_cache_root = crosspm.get('cache', '').strip().strip('"').strip("'")
@@ -497,7 +573,7 @@ class Config(object):
             home_dir = os.getenv('APPDATA') if WINDOWS else os.getenv('HOME')
             self.crosspm_cache_root = os.path.join(home_dir, '.crosspm')
         self.crosspm_cache_root = os.path.realpath(os.path.expanduser(self.crosspm_cache_root))
-        self.cache = cache_config
+        self.cache_config = cache_config
 
     def init_not_columns(self):
         # gather items from options
@@ -515,14 +591,15 @@ class Config(object):
         if 'common' not in parsers:
             parsers['common'] = {}
         for k, v in parsers.items():
-            if k not in self._parsers:
-                v.update({_k: _v for _k, _v in parsers['common'].items() if _k not in v})
-                self._parsers[k] = Parser(k, v, self)
-            else:
-                code = CROSSPM_ERRORCODE_CONFIG_FORMAT_ERROR
-                msg = 'Config file contains multiple definitions of the same parser: [{}]'.format(k)
-                self._log.exception(msg)
-                raise CrosspmException(code, msg)
+            if k != 'common':
+                if k not in self._parsers:
+                    v.update({_k: _v for _k, _v in parsers['common'].items() if _k not in v})
+                    self._parsers[k] = Parser(k, v, self)
+                else:
+                    code = CROSSPM_ERRORCODE_CONFIG_FORMAT_ERROR
+                    msg = 'Config file contains multiple definitions of the same parser: [{}]'.format(k)
+                    self._log.exception(msg)
+                    raise CrosspmException(code, msg)
         if len(self._parsers) == 0:
             code = CROSSPM_ERRORCODE_CONFIG_FORMAT_ERROR
             msg = 'Config file does not contain parsers! Unable to process any further.'
@@ -560,7 +637,7 @@ class Config(object):
             try:
                 _temp = __import__(_app_name, globals(), locals(), ['setup', 'Adapter'], 0)
                 _names = _temp.setup['name']
-                if type(_names) is str:
+                if isinstance(_names, str):
                     _names = [_names]
                 self._adapters.update({k: _temp.Adapter(self) for k in _names if k in types})
 
@@ -576,12 +653,12 @@ class Config(object):
     def parse_options(options, cmdline, check_default=False):
         # try:
         #     _cmdline = {x[0]: x[1] for x in [x.strip().split('=') for x in cmdline.split(',')] if len(x) > 1}
-        # except:
+        # except Exception:
         #     _cmdline = {}
         _remove = []
         for k, v in options.items():
             # if option type is str, leave the value intact
-            if type(options[k]) is not str:
+            if not isinstance(options[k], (str, bool)):
                 # if option has cmdline name, try to fetch a value from command line by cmdline name
                 if 'cmdline' in v:
                     if v['cmdline'] in cmdline:
@@ -592,7 +669,7 @@ class Config(object):
                     options[k] = cmdline[k]
 
                 # if option is still not filled, try to get a value from environment by env name
-                if type(options[k]) is not str:
+                if not isinstance(options[k], str):
                     success = False
                     if 'env' in v:
                         _env = os.getenv(v['env'])
@@ -633,10 +710,11 @@ class Config(object):
     def get_columns(self):
         return self._columns
 
-    def complete_params(self, _vars):
-        _vars.update({k: v for k, v in self._not_columns.items() if k not in _vars})
-        _vars.update({k: '' for k in self._columns if k not in _vars})
-        return _vars
+    def complete_params(self, _vars, update=True):
+        result = _vars if update else {}
+        result.update({k: v for k, v in self._not_columns.items() if k not in _vars})
+        result.update({k: '' for k in self._columns if (k not in _vars) and (k not in result)})
+        return result
 
     def get_values(self, column_name):
         _res = None
@@ -686,7 +764,7 @@ class Config(object):
 
         default = None
         for x in levels:
-            if type(level) is str:
+            if isinstance(level, str):
                 if x[0] == level.lower():
                     return x[1]
             elif x[2]:
@@ -704,7 +782,7 @@ class Config(object):
                 result = _temp.GetSystemDirectory()
                 if result:
                     result = os.path.splitdrive(result)[0]
-            except:
+            except Exception:
                 pass
         if not result:
             result = 'C:'
