@@ -1,21 +1,30 @@
 # -*- coding: utf-8 -*-
 
+import json
 import logging
 import os
 import re
-import json
+import sys
+
+from jinja2 import Environment, FileSystemLoader
+from collections import OrderedDict
+from typing import Dict
+
 from crosspm.helpers.exceptions import *
 from crosspm.helpers.parser import Parser
+from crosspm.helpers.config import Config
+from crosspm.helpers.package import Package
+
+_output_format_map = {}  # Contain map for output format, load from decorator
 
-_output_format_map = {}
 (
     PLAIN,
     DICT,
-    LIST,
+    LIST
 ) = range(3)
 
 
-class OutFormat(object):
+class OutFormat:
     def __init__(self, value, esc_path=False):
         self._value = value
         self._esc_path = esc_path
@@ -47,6 +56,10 @@ class OutFormat(object):
 
 
 def register_output_format(name):
+    """
+    Load output format function to dictionary (decorator with this function name)
+    """
+
     def check_decorator(fn):
         _output_format_map[name] = fn
 
@@ -58,8 +71,8 @@ def register_output_format(name):
     return check_decorator
 
 
-class Output(object):
-    _config = {
+class Output:
+    _output_config = {
         'root': {'PACKAGES_ROOT'},
         # 'key': 'package',
         'value': 'path',
@@ -79,16 +92,17 @@ class Output(object):
 
     def __init__(self, data=None, name_column='', config=None):
         self._log = logging.getLogger('crosspm')
+        self._config: Config = config
         if name_column:
             self._name_column = name_column
-            self._config['key'] = name_column
+            self._output_config['key'] = name_column
         if data and isinstance(data, dict):
-            self._config = data
+            self._output_config = data
         # self.init_config()
-        if 'columns' not in self._config:
-            self._config['columns'] = []
+        if 'columns' not in self._output_config:
+            self._output_config['columns'] = []
 
-        if len(self._config['columns']) == 0:
+        if len(self._output_config['columns']) == 0:
             # TODO: implement this:
             # if config.no_fails:
             #     param_list = [x for x in config.get_fails('unique', {})]
@@ -98,7 +112,7 @@ class Output(object):
             # else:
             pkg_name = '{:upper}_ROOT'
 
-            self._config['columns'] = [
+            self._output_config['columns'] = [
                 {
                     'column': self._name_column,
                     'value': pkg_name,
@@ -110,27 +124,27 @@ class Output(object):
             ]
 
         self.init_config()
-        if 'columns' not in self._config:
-            self._config['columns'] = []
+        if 'columns' not in self._output_config:
+            self._output_config['columns'] = []
 
     def init_config(self):
-        root = self._config.get('root', '')
+        root = self._output_config.get('root', '')
         if isinstance(root, str):
-            self._config['type'] = PLAIN
+            self._output_config['type'] = PLAIN
         elif isinstance(root, (dict, set)):
-            self._config['type'] = DICT
+            self._output_config['type'] = DICT
             root = [x for x in root]
-            self._config['root'] = root[0] if len(root) > 0 else ''
+            self._output_config['root'] = root[0] if len(root) > 0 else ''
         elif isinstance(root, (list, tuple)):
-            self._config['type'] = LIST
-            self._config['root'] = root[0] if len(root) > 0 else ''
+            self._output_config['type'] = LIST
+            self._output_config['root'] = root[0] if len(root) > 0 else ''
 
-        key = self._config.get('key', '')
+        key = self._output_config.get('key', '')
         key0 = None
 
-        if 'columns' in self._config:
+        if 'columns' in self._output_config:
             self._columns = []
-            for item in self._config['columns']:
+            for item in self._output_config['columns']:
                 if not item.get('value', ''):
                     item['value'] = '{}'
                 if not item.get('name', ''):
@@ -151,7 +165,7 @@ class Output(object):
 
         if key0:
             key = key0
-        self._config['key'] = key
+        self._output_config['key'] = key
 
         if self._columns:
             if self._name_column and self._name_column not in self._columns:
@@ -159,15 +173,15 @@ class Output(object):
         else:
             if not self._name_column:
                 self._name_column = 'package'
-        if 'value' not in self._config:
-            self._config['value'] = ''
+        if 'value' not in self._output_config:
+            self._output_config['value'] = ''
 
-    def get_var_name(self, pkg_name):
+    @staticmethod
+    def get_var_name(pkg_name):
         result = '{:upper.safe}'.format(OutFormat(pkg_name))
         return result
 
     def write_to_file(self, text, out_file_path):
-        # out_file_path = os.path.realpath(os.path.expanduser(out_file_path))
         out_dir_path = os.path.dirname(out_file_path)
 
         if not os.path.exists(out_dir_path):
@@ -188,30 +202,32 @@ class Output(object):
     def get_output_types():
         return list(_output_format_map.keys())
 
-    def write(self, params, packages):
+    def write_output(self, params, packages: Dict[str, Package]):
+        """
+        Функция вызывает определенную функцию для фиксированного out-format
+        :param params:
+        :param packages:
+        :return:
+        """
         if params['out_format'] not in _output_format_map:
             raise CrosspmException(
                 CROSSPM_ERRORCODE_UNKNOWN_OUT_TYPE,
-                'unknown output type: [{}]'.format(params['out_format']),
+                'Unknown out_format: [{}]'.format(params['out_format']),
             )
 
         f = _output_format_map[params['out_format']]
-        result = f(self, packages)
+        result = f(self, packages, **params)
 
         if result:
             out_file_path = os.path.realpath(os.path.expanduser(params['output']))
-            # out_dir = os.path.dirname(out_file_path)
-            # if not os.path.exists(out_dir):
-            #     os.makedirs(out_dir)
             self.write_to_file(result, out_file_path)
+            msg = 'Writing lock file' if params['out_format'] == 'lock' else 'Write packages info to file'
             self._log.info(
-                'Write packages info to file [%s]\ncontent:\n\n%s',
-                out_file_path,
-                result,
+                f'{msg} {out_file_path}\ncontent:\n\n{result}',
             )
 
     def format_column(self, column, name, value):
-        for item in self._config['columns']:
+        for item in self._output_config['columns']:
             if item['column'] == column:
                 name = item['name'].format(OutFormat(name))
                 value = item['value'].format(OutFormat(value))
@@ -219,74 +235,138 @@ class Output(object):
         return {'name': name, 'value': value}
 
     @register_output_format('stdout')
-    def output_type_stdout(self, packages):
-        self._config['type'] = PLAIN
-        for k, v in self.output_type_module(packages).items():
+    def output_format_stdout(self, packages, **kwargs):
+        self._output_config['type'] = PLAIN
+        _packages = self.output_format_module(packages)
+        for k in _packages:
+            v = _packages[k]
             sys.stdout.write('{}: {}\n'.format(self.get_var_name(k), v))
             sys.stdout.flush()
         return None
 
     @register_output_format('shell')
-    def output_type_shell(self, packages):
-        self._config['type'] = PLAIN
+    def output_format_shell(self, packages, **kwargs):
+        self._output_config['type'] = PLAIN
         result = '\n'
-        for k, v in self.output_type_module(packages).items():
+        _packages = self.output_format_module(packages)
+        for k in _packages:
+            v = _packages[k]
             result += "{}='{}'\n".format(self.get_var_name(k), v)
         result += '\n'
         return result
 
+    @register_output_format('jinja')
+    def output_format_jinja(self, packages, output_template, **kwargs):
+        output_template = os.path.realpath(output_template)
+        template_dir, template_name = os.path.split(output_template)
+        j2_env = Environment(loader=FileSystemLoader(template_dir), trim_blocks=True)
+        result = j2_env.get_template(template_name).render(packages=packages)
+        return result
+
     @register_output_format('cmd')
-    def output_type_cmd(self, packages):
-        self._config['type'] = PLAIN
+    def output_format_cmd(self, packages, **kwargs):
+        self._output_config['type'] = PLAIN
         result = '\n'
-        for k, v in self.output_type_module(packages).items():
+        _packages = self.output_format_module(packages)
+        for k in _packages:
+            v = _packages[k]
             result += "set {}={}\n".format(self.get_var_name(k), v)
         result += '\n'
         return result
 
-    # @register_output_format('module')
-    def output_type_module(self, packages, esc_path=False):
-        result_list = []
-        for _pkg in packages.values():
-            if _pkg:
-                _pkg_params = _pkg.get_params(self._columns, True)
-                _res_item = {}
-                for item in self._config['columns']:
-                    name = item['name'].format(OutFormat(item['column']))
-                    value = _pkg_params.get(item['column'], '')
-                    if not isinstance(value, (list, dict, tuple)):
-                        try:
-                            value = item['value'].format(
-                                OutFormat(value, (item['column'] == 'path') if esc_path else False))
-                        except:
-                            value = ''
-                    # TODO: implement this:
-                    # if not value:
-                    #     try:
-                    #         value = item['value'].format(OutFormat(_pkg.get_params('', True)))
-                    #     except:
-                    #         pass
-                    _res_item[name] = value
-                result_list.append(_res_item)
+    @register_output_format('lock')
+    def output_format_lock(self, packages, **kwargs):
+        """ Text to lock file """
+        self._output_config['type'] = PLAIN
+        text = ''
+        tmp_packages = OrderedDict()
+        columns = self._config.get_columns()
+        widths = {}
+        _packages = []
 
-        if self._config['type'] == LIST:
+        for _package in packages.values():
+            if _package:
+                _packages.append(_package)
+                _packages += _package.all_packages
+
+        for _pkg in _packages:
+            _pkg_name = _pkg.package_name
+            _params = _pkg.get_params(columns, merged=True, raw=False)
+            if _pkg_name not in tmp_packages:
+                tmp_packages[_pkg_name] = _params
+                comment = 1
+                for _col in columns:
+                    widths[_col] = max(widths.get(_col, len(_col)), len(str(_params.get(_col, '')))) + comment
+                    comment = 0
+        comment = 1
+        for _col in columns:
+            text += '{}{} '.format(_col, ' ' * (widths[_col] - len(_col) - comment))
+            comment = 0
+        text = '#{}\n'.format(text.strip())
+        for _pkg_name in sorted(tmp_packages, key=lambda x: str(x).lower()):
+            _pkg = tmp_packages[_pkg_name]
+            line = ''
+            for _col in columns:
+                line += '{}{} '.format(_pkg[_col], ' ' * (widths[_col] - len(str(_pkg[_col]))))
+            text += '{}\n'.format(line.strip())
+        return text
+
+    def output_format_module(self, packages, esc_path=False):
+        """
+        Create out with child first position
+        """
+
+        def create_ordered_list(packages_):
+            """
+            Recursive for package.packages
+            """
+            list_ = []
+            for _pkg_name in packages_:
+                _pkg = packages_[_pkg_name]
+                if _pkg and _pkg.packages:
+                    list_.extend(create_ordered_list(_pkg.packages))
+                if _pkg:
+                    _pkg_params = _pkg.get_params(self._columns, True)
+                    _res_item = {}
+                    for item in self._output_config['columns']:
+                        name = item['name'].format(OutFormat(item['column']))
+                        value = _pkg_params.get(item['column'], '')
+                        if not isinstance(value, (list, dict, tuple)):
+                            try:
+                                value = item['value'].format(
+                                    OutFormat(value, (item['column'] == 'path') if esc_path else False))
+                            except Exception:
+                                value = ''
+                        # TODO: implement this:
+                        # if not value:
+                        #     try:
+                        #         value = item['value'].format(OutFormat(_pkg.get_params('', True)))
+                        #     except Exception as e:
+                        #         pass
+                        _res_item[name] = value
+                    list_.append(_res_item)
+            return list_
+
+        result_list = create_ordered_list(packages, )
+
+        if self._output_config['type'] == LIST:
             return result_list
 
-        result = {}
+        result = OrderedDict()
         for item in result_list:
             # TODO: Error handling
-            name = item[self._config['key']]
-            if self._config['value']:
-                value = item[self._config['value']]
+            name = item[self._output_config['key']]
+            if self._output_config['value']:
+                value = item[self._output_config['value']]
             else:
-                value = {k: v for k, v in item.items() if k != self._config['key']}
+                value = OrderedDict([(k, v) for k, v in item.items() if k != self._output_config['key']])
 
             result[name] = value
 
         return result
 
     @register_output_format('python')
-    def output_type_python(self, packages):
+    def output_format_python(self, packages, **kwargs):
         def get_value(_v):
             if isinstance(_v, (int, float, bool)):
                 _res = '{}'.format(str(_v))
@@ -296,16 +376,24 @@ class Output(object):
                 _res = "'{}'".format(str(_v))
             return _res
 
+        def items_iter(_dict_or_list):
+            if isinstance(_dict_or_list, dict):
+                for k in sorted(_dict_or_list, key=lambda x: str(x).lower()):
+                    yield k
+            else:
+                for k in _dict_or_list:
+                    yield k
+
         result = '# -*- coding: utf-8 -*-\n\n'
         res = ''
-        dict_or_list = self.output_type_module(packages, True)
-        for item in dict_or_list:
-            if self._config['type'] == LIST:
+        dict_or_list = self.output_format_module(packages, True)
+        for item in items_iter(dict_or_list):
+            if self._output_config['type'] == LIST:
                 res += '    {\n'
                 for k, v in item.items():
                     res += "        '{}': {},\n".format(k, get_value(v))
                 res += '    },\n'
-            elif self._config['type'] == DICT:
+            elif self._output_config['type'] == DICT:
                 res += "    '{}': ".format(item)
                 if isinstance(dict_or_list[item], dict):
                     res += '{\n'
@@ -320,7 +408,7 @@ class Output(object):
                 else:  # str
                     res += "{},\n".format(get_value(dict_or_list[item]))
             else:
-                res += '{} = '.format(self.get_var_name(item), get_value(dict_or_list[item]))
+                res += '{} = '.format(self.get_var_name(item))  # , get_value(dict_or_list[item]))
                 if isinstance(dict_or_list[item], dict):
                     res += '{\n'
                     for k, v in dict_or_list[item].items():
@@ -334,12 +422,12 @@ class Output(object):
                 else:  # str
                     res += "{}\n".format(get_value(dict_or_list[item]))
 
-        if self._config['type'] == LIST:
-            result += '{} = [\n'.format(self._config['root'] or 'PACKAGES_ROOT')
+        if self._output_config['type'] == LIST:
+            result += '{} = [\n'.format(self._output_config['root'] or 'PACKAGES_ROOT')
             result += res
             result += ']\n'
-        elif self._config['type'] == DICT:
-            result += '{} = {}\n'.format(self._config['root'] or 'PACKAGES_ROOT', '{')
+        elif self._output_config['type'] == DICT:
+            result += '{} = {}\n'.format(self._output_config['root'] or 'PACKAGES_ROOT', '{')
             result += res
             result += '}\n'
         else:
@@ -348,11 +436,11 @@ class Output(object):
         return result
 
     @register_output_format('json')
-    def output_type_json(self, packages):
-        dict_or_list = self.output_type_module(packages)
-        if self._config['root']:
+    def output_format_json(self, packages, **kwargs):
+        dict_or_list = self.output_format_module(packages)
+        if self._output_config['root']:
             dict_or_list = {
-                self._config['root']: dict_or_list
+                self._output_config['root']: dict_or_list
             }
         # TODO: Find a proper way to avoid double backslashes in path only (or not?)
         result = json.dumps(dict_or_list, indent=True)  # .replace('\\\\', '\\')
